"""Special module, which adds some functionality to enum class."""
from enum import Enum
from typing import List


class EnhancedEnum(Enum):
    """
    Enhanced enum class, which adds methods for getting enum name and values.
    Used in enum validator method in utils.py.

    This helps to get rid of parameters that require an enum pass.
    Now user can pass simple strings, which will then be compared
    with the values of the required enum.
    """

    @classmethod
    def get_name(cls) -> str:
        """Returns name of the enum."""
        return cls.__name__

    @classmethod
    def get_values(cls) -> List[str]:
        """Returns values of the enum."""
        return list(map(lambda c: c.value, cls))



"""Enums for /api/favorites."""
from shikithon.enums.enhanced_enum import EnhancedEnum


class FavoriteLinkedType(EnhancedEnum):
    """Contains constants related for favorite linked type."""
    ANIME = 'Anime'
    MANGA = 'Manga'
    RANOBE = 'Ranobe'
    PERSON = 'Person'
    CHARACTER = 'Character'



"""Enums for /api/styles."""
from shikithon.enums.enhanced_enum import EnhancedEnum


class OwnerType(EnhancedEnum):
    """Contains constants related for style owner type."""
    USER = 'User'
    CLUB = 'Club'



"""Custom exceptions for API class."""


class MissingConfigData(Exception):

    def __init__(self):
        super().__init__('It is impossible to initialize an API object'
                         'without missing variables. '
                         'Recheck your config and try again.')


class MissingAppVariable(Exception):

    def __init__(self, variable_name: str):
        super().__init__(f'It is impossible to initialize an API object'
                         f'without missing variable "{variable_name}". '
                         f'Recheck your config and try again.')


class MissingAuthCode(Exception):

    def __init__(self, auth_link: str):
        super().__init__('It is impossible to initialize an API object'
                         'without missing auth code. To get one, go to '
                         f'{auth_link} and insert code in config.')


class AccessTokenException(Exception):

    def __init__(self, error_message: str):
        super().__init__(
            'An error occurred while receiving tokens, '
            f'here is the information from the response: {error_message}')
